module Iowa

	# NLS provides a container for storing and retrieving mappings of
	# language/label to text.
	
	class NLS
		attr_accessor :default_lang
		
		# Take a label such as EN-US and convert it to en_us
		
		def normalize_lang(lang)
			lang.gsub(/-/,'_').downcase
		end
		
		# Initialize the container.  The container assumes that the default
		# language is English, though another default can be passed to
		# initialize. i.e. nls = NLS.new('es').
		# The default language code can be passed as a string or a symbol.
		
		def initialize(lang = 'en')
			@default_lang = normalize_lang(lang.to_s)
			@data = {}
			@data[@default_lang} = {}
			@partial_to_full = {}
		end
		
		def default_lang=(val)
			@default_lang = normalize_lang(lang.to_s)
		end
		
		def has_text?(label,lang = nil)
			label = label.to_s
			lang = normalize_lang(lang.to_s) || @default_lang
			r = lambda {
				if @data.has_key?(lang)
					if @data[lang].has_key?(label)
						break lang,label
					end
				end
				
				partial = lang.split('_',2).first
				if partial != lang and @data.has_key?(partial)
					if @data[partial].has_key?(label)
						break partial,label
					end
				end
				
				# If it falls through to here, then we don't have any good matches
				# for the language.  The next thing to try is to see if there's
				# anything in the language family that will work.
				# I don't want to be doing full scans of the structure to gather
				# language keys for each request that falls to this point, though,
				# so the code will cache the language keys for a particular partial.
				# It'll then take the first one (alphabetically) that matches the partial.
				unless @partial_to_full.has_key?(partial)
					m = []
					rxp = Regexp.new("^#{partial}")
					@data.each_key do |k|
						next if partial == k
						m.push k if rxp === k
					end
					@partial_to_full[partial] = m
				end
				
				@partial_to_full[partial].each do |l|
					if @data[l].has_key?(label)
						break l,label
					end
				end
				
				if @data[@default_lang].has_key?(label)
					break @default_lang,label
				end
				
				# If execution falls through to here, there were no matches.
				break false
			}.call

			r
		end
		
		# Retrieve a piece of text for the given label and language.  If language
		# is omitted, the default is used.  Called like:  nls[:intro, :en]
		
		def [](label,lang = nil)
			abel,ang = has_text?(label,lang)
			abel ? @data[abel][ang] : '__undefined__'
		end
		alias :[], :get_text
		
		# Sets a piece of text in the collection for a given label/language.
		
		def []=(label,mv1,mv2 = :__NO_ARG__)
			if mv2 == :__NO_ARG__
				lang = @default_lang
				text = mv1
			else
				lang = normalize_lang(mv1)
				text = mv2
			end
			@data[lang] = {} unless @data.has_key?(lang)
			@data[lang][label] = text
		end
	end
end